#ifndef QCOLORMAPACTION_H
#define QCOLORMAPACTION_H

#include <QAction>
#include <vector>
#include <cmath>

using namespace std;

/**
 * @brief Derivation of QAction to create color maps.
 * @author Ruediger Knoerig
 */
class QColorMapAction : public QAction
{
public:
    QColorMapAction(QString name,QObject *parent) : QAction(name,parent),mapname(name) { setCheckable(true); }

    ~QColorMapAction() {}

    /**
     * @brief Returns the name of the colormap.
     * @return Name of the colormap
     */
    QString getName() const { return mapname; }

    /**
     * @brief Calculating a colormap.
     * Calculates the colormap in the vector colormap.
     * @param colormap Vector for the colormap entries
     */
    virtual void apply(vector<QRgb> &colormap) = 0;

protected:
     const QString mapname; /**< name of the colormap */

     /**
      * @brief This class provides a gaussian-like distribution with a maximum of 255.
      * @author Ruediger Knoerig
      */
     class GaussDist
     {
     public:
         /**
          * @brief Constructor.
          * @param mu mean value
          * @param sigma standard deviation alike
          */
         GaussDist(int mu,double sigma)
         {
            this->mu=mu;
            scale=-1.0/(sigma*sigma);
         }

         /**
          * @brief Calculation operator.
          * @param x input value
          * @return \f$255\cdot e^{-frac{(x-\mu)^2}{\sigma^2}}\f$
          */
         int operator()(int x)
         {
             x-=mu;
             x*=x;
            return int(255.0*exp(scale*x));
         }
     protected:
         double prefactor;
         double scale;
         int mu;
     };
};

#endif // QCOLORMAPACTION_H
