#include "qfraktalview.h"
#include <QString>
#include <QtDebug>
#include <QIcon>
#include "defaultcolormap.h"
#include "rainbow_soft.h"
#include "rainbow_narrow.h"
#include "rainbow_wide.h"
#include "rainbow_linear.h"
#include "SpectralMap.h"
#include "LinearGrayMap.h"
#include "LogGrayMap.h"

void QFraktalView::renderFractal()
{
   setCursor(Qt::WaitCursor);
   isRendering=true;
   threadNumActions->setEnabled(false);
   bool aa=antialias->isChecked();
   antialias->setEnabled(false);
   stopwatch.start();
   activeRenderers=usedRenderers;
   double square=sqrt(double(usedRenderers));
   int ydivs=ceil(square);
   int xdivs=floor(square);
   QRect region=fraktal.rect();
   region.setWidth(region.width()/xdivs);
   region.setHeight(region.height()/ydivs);
   int rendc=0;
   for(int yc=0;yc<ydivs;yc++)
   {
    for(int xc=0;xc<xdivs;xc++)
        {
            //qDebug() << "xc: " << xc << " yc: " << yc << "region: " << region;
            renderer[rendc++].render(ul,lr,dr,di,colormap,region,aa);
            region.moveLeft(region.left()+region.width());
        }
        region.moveLeft(0);
        region.moveBottom(region.bottom()+region.height());
   }
}

void QFraktalView::mousePressEvent(QMouseEvent *e)
{
    switch(e->button())
    {
        case Qt::LeftButton: // draw zoom selection
            selection.setBottomLeft(e->pos());
            break;
        case Qt::RightButton: // show context menu
            break;
        default:
            break;
    }
}

void QFraktalView::mouseMoveEvent(QMouseEvent *e)
{
    if(e->buttons() == Qt::LeftButton)
    {
        // draw zoom selection
        selection.setTopRight(e->pos());
        update();
    }
    QPoint p=e->pos();
    complex<double> c=ul+complex<double>(p.x()*dr,p.y()*di);
    QString txt=QString("%1+i*%2").arg(c.real()).arg(c.imag());
    setToolTip(txt);
    emit signalStatusChanged(txt);
}

void QFraktalView::mouseReleaseEvent(QMouseEvent *e)
{
    if(e->button() == Qt::LeftButton)
    {
        int xmin=selection.left();
        int xmax=selection.right();
        if(xmin > xmax) qSwap(xmin,xmax);
        int ymin=selection.top();
        int ymax=selection.bottom();
        if(ymin > ymax) qSwap(ymin,ymax);
        complex<double> nul=ul+complex<double>(xmin*dr,ymin*di);
        complex<double> nlr=ul+complex<double>(xmax*dr,ymax*di);
        slotSetView(nul,nlr);
        selection = QRect();
    }
}

void QFraktalView::initActions()
{
    contextmenu = new QMenu(this);
    resetZoom = contextmenu->addAction(QIcon(":/icons/zoomReset.png"),tr("Reset Zoom"),this,SLOT(slotResetView()),QKeySequence("Home"));
    resetZoom->setToolTip(tr("Reset to view whole mandelbrot set."));
    zoomIn = contextmenu->addAction(QIcon(":/icons/zoomin.png"),tr("Zoom in"),this,SLOT(slotZoomIn()),QKeySequence::ZoomIn);
    zoomOut = contextmenu->addAction(QIcon(":/icons/zoomout.png"),tr("Zoom out"),this,SLOT(slotZoomOut()),QKeySequence::ZoomOut);
    colormapActions = new QActionGroup(this);
    QAction *defaultmap=(QAction *) new DefaultColorMap(this);
    colormapActions->addAction(defaultmap);
    colormapActions->addAction((QAction *) new SpectralMap(this));
    colormapActions->addAction((QAction *) new Rainbow_soft(this));
    colormapActions->addAction((QAction *) new Rainbow_narrow(this));
    colormapActions->addAction((QAction *) new Rainbow_wide(this));
    colormapActions->addAction((QAction *) new Rainbow_linear(this));
    colormapActions->addAction((QAction *) new LinearGrayMap(this));
    colormapActions->addAction((QAction *) new LogGrayMap(this));
    colormapActions->setExclusive(true);
    defaultmap->setChecked(true);
    connect(colormapActions,SIGNAL(triggered(QAction*)),this,SLOT(slotColormapChanged(QAction*)));
    QMenu *viewColorMapMenu=new QMenu(tr("Colormaps"));
    viewColorMapMenu->addActions(colormapActions->actions());
    contextmenu->addMenu(viewColorMapMenu);
    threadNumActions=new QActionGroup(this);
    threadNumActions->setExclusive(true);
    for(int threads=1;threads<=numThreads;threads<<=1)
    {
        QAction *a=new RenderNumAction(threads,this);
        if(threads==usedRenderers) a->setChecked(true);
        threadNumActions->addAction(a);
        connect(a,SIGNAL(triggered()),this,SLOT(slotSetNumThreads()));
    }
    antialias = new QAction(tr("Anti-alias"),this);
    antialias->setCheckable(true);
    antialias->setChecked(true);
    connect(antialias,SIGNAL(triggered()),this,SLOT(renderFractal()));
}
